/*******************************************************************************

Copyright (c) 2003, Robert Cowham and Vaccaperna Systems Ltd.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL VACCAPERNA SYSTEMS LTD. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/*******************************************************************************
 * Name:	TraceUtils.h
 *
 * Author:	Robert Cowham <robert@vaccaperna.co.uk>
 *
 * Description: 
 *		A Tracing utility
 *
 ******************************************************************************/

#ifndef _TRACEUTILS_H_
#define _TRACEUTILS_H_

#include "stdafx.h"
#include "OleType.h"
#include <stdio.h>

BOOL Tracing();

inline void OutputTraceMsg(LPCTSTR msg)
{
	OutputDebugString(msg);
}

// This function is stolen from AtlTrace which gets #ifdef ed out if not in Debug mode...
inline void FormatTrace(LPCTSTR lpszFormat, ...)
{
	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	TCHAR szBuffer[512];

	nBuf = _vsntprintf(szBuffer, sizeof(szBuffer), lpszFormat, args);
	ATLASSERT(nBuf < sizeof(szBuffer)); //Output truncated as it was > sizeof(szBuffer)

	OutputDebugString(szBuffer);
	va_end(args);
}

inline void OutputTraceMsg(LPCTSTR msg, LPCTSTR param)
{
	FormatTrace(_T("%s: %s"), msg, param);
}

inline void LogException(LPCTSTR proc)
{
	FormatTrace(_T("p4vb exception in procedure %s"), proc);
}


#define TRACE(_Msg) if (Tracing()) { OutputTraceMsg(_Msg); }
#define TRACE1(_Msg, Param1) if (Tracing()) { OutputTraceMsg(_Msg, Param1); }

class Tracker  
{
public:
	inline Tracker(LPCTSTR Stack)
	{
		TRACE1(_T("p4vb: Enter"), Stack);
		if (Tracing())	m_Procname = Stack;
	}

	virtual ~Tracker()
	{
		TRACE1(_T("p4vb: Leave"), m_Procname);
	}

private:
	String m_Procname;
};

#define STACK_TRACE(_Proc) Tracker _MyTracker(_T(_Proc));

#endif // _TRACEUTILS_H_
