/*******************************************************************************

Copyright (c) 2003, Robert Cowham and Vaccaperna Systems Ltd.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL VACCAPERNA SYSTEMS LTD. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/*******************************************************************************
 * Name:	p4clientapi.h
 *
 * Author:	Robert Cowham <robert@vaccaperna.co.uk>
 *
 * Description: 
 *		COM bindings for the Perforce API. User interface class
 * 		for getting Perforce results into COM.
 *
 ******************************************************************************/

#include "oletype.h"

#ifndef _P4API_H_
#define _P4API_H_

#ifdef SetPort		// Windows bug
#undef SetPort
#endif

// P4 API files
# include "clientapi.h"
# include "enviro.h"
# include "signaler.h"

# include "msgclient.h"
# include "spec.h"

#endif // _P4API_H_

#include "p4clientuser.h"

class p4ClientApi {

public:
	// caller's main interface

	p4ClientApi();
	~p4ClientApi();

	    // Protocol options. Call before Connect()
    void Tagged();
    void ParseForms();

    // Session management
    int Connect(String &errMsg);
    int Disconnect();

	void  ExceptionLevel( int i )	{ m_exceptionLevel = i; }
    int   ExceptionLevel()		{ return m_exceptionLevel; }

	Long Run(String cmd, String &errMsg);
	void p4ClientApi::CheckCharset();

	void SetCharset(String s)
	{
		m_client.SetCharset(ConvertString(s));
	}
	void SetClient(String s)
	{
		m_client.SetClient(ConvertString(s));
	}
	void SetCwd(String s)
	{
		m_client.SetCwd(ConvertString(s));
	}
	void SetHost(String s)
	{
		m_client.SetHost(ConvertString(s));
	}
	void SetLanguage(String s)
	{
		m_client.SetLanguage(ConvertString(s));
	}
	void SetPassword(String s)
	{
		m_client.SetPassword(ConvertString(s));
	}
	void SetPort(String s)
	{
		m_client.SetPort(ConvertString(s));
	}
	void SetUser(String s)
	{
		m_client.SetUser(ConvertString(s));
	}

	const String GetCharset() 
	{
		String result(m_client.GetCharset().Text());
		return result;
	}
	const String GetClient() 
	{
		String result(m_client.GetClient().Text());
		return result;
	}
	const String GetCwd()
	{
		String result(m_client.GetCwd().Text());
		return result;
	}
	const String GetHost()
	{
		String result(m_client.GetHost().Text());
		return result;
	}
	const String GetLanguage()
	{
		String result(m_client.GetLanguage().Text());
		return result;
	}
	const String GetOs()
	{
		String result(m_client.GetOs().Text());
		return result;
	}
	const String GetPassword()
	{
		String result(m_client.GetPassword().Text());
		return result;
	}
	const String GetPort()
	{
		String result(m_client.GetPort().Text());
		return result;
	}
	const String GetUser()
	{
		String result(m_client.GetUser().Text());
		return result;
	}


public:
	p4ClientUser m_ui;

private:
	ClientApi m_client;
    int m_initCount;
    int m_exceptionLevel;
    int	m_server2;
    int	m_tagged;
	StrBuf m_emptyBuf;
	StrBuf m_buf;

private:
	StrBuf *ConvertString(String &s)
	{
		if (s.IsEmpty())
		{
			return &m_emptyBuf;
		}
		else
		{
			m_buf = (LPCSTR)s;
			return &m_buf;
		}
	}

} ;
